@@ -24,12 +24,29 @@
 #define DIRECTORY struct dirent
 #endif
 #include <errno.h>
+#include <sys/stat.h>
 #ifdef IMA_SYMM
 #include <strings.h>
 #else
 #include <string.h>
 #endif
-#include <sys/stat.h>
+/*
+ * FIX for brain-damage induced by IBM and others who insist on
+ * using #define to alias index to strchr, etc.
+ */
+#ifdef rindex
+#undef rindex
+#endif
+#ifdef index
+#undef index
+#endif
+#ifdef strchr
+#undef strchr
+#endif
+#ifdef strrchr
+#undef strrchr
+#endif
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Form.h>
@@ -53,6 +70,13 @@
 #endif /*IMA_PMAX*/
 #endif /*R_OK*/
 
+
+#ifdef MAX
+#undef MAX
+#endif
+#ifdef MIN
+#undef MIN
+#endif
 #define MAX(a,b) (((int)(a) > (int)(b)) ? (a) : (b))
 #define MIN(a,b) (((int)(a) < (int)(b)) ? (a) : (b))
 
@@ -104,7 +128,9 @@
 #endif
 #endif
 
+#ifndef errno
 extern int errno;				/* 4.3bsd needs this */
+#endif
 
 /****************************************************************
  *
@@ -348,6 +374,7 @@ FileselectWidget w;
 		XawTextPosition pos;
 		char *basename, *dirname;
 		char *ptr;
+		char *rindex ();
 			
 		dirname = XtNewString (w->fileselect.selection);
 		ptr = rindex (w->fileselect.selection, '/');
@@ -556,11 +583,9 @@ XEvent *event;
 String *params;
 Cardinal *num_params;
 {
-	char *sel;
 	String currentVal;
 	XawTextPosition begin, end;
 	char bigbuf[1024];
-	int len;
 	char *src, *dst, *endbuf;
 	FileselectWidget fw = (FileselectWidget) XtParent(w);
 
@@ -611,7 +636,8 @@ Cardinal *num_params;
 	FileselectWidget parent = (FileselectWidget) XtParent (w);
 	/* Widget parent_popup = XtParent(XtParent(w)); */
 
-	XtCallCallbackList (parent, parent->fileselect.callbacks, (XtPointer) 1);
+	XtCallCallbackList ((Widget) parent, parent->fileselect.callbacks,
+						(XtPointer) 1);
 	/* XtDestroyWidget( parent_popup ); */
 }
 
@@ -639,6 +665,7 @@ char *name;
 		}
 		else {
 			char dirname[1024];
+			char *rindex ();
 			char *ptr;
 
 			strcpy (dirname, name);
@@ -674,12 +701,11 @@ XEvent *event;
 String *params;
 Cardinal *num_params;
 {
-	Arg args[20];
 	String currentVal;
 	FileselectWidget fw = (FileselectWidget) XtParent(w);
 	char buf[1025];				/* XXX should be MAXPATHNAME or some such */
 	char *ptr;
-	int length;
+	char *index ();
 
 	XtVaGetValues (fw->fileselect.currSelectText,
 				   XtNstring, &currentVal,
@@ -707,7 +733,7 @@ Cardinal *num_params;
 
 	fw->fileselect.selection = XtNewString (buf);
 
-	XtCallCallbackList (fw, fw->fileselect.callbacks, (XtPointer) 0);
+	XtCallCallbackList ((Widget) fw, fw->fileselect.callbacks, (XtPointer) 0);
 }
 
 static void
@@ -734,7 +760,6 @@ static void
 Initialize (request, new)
 Widget request, new;
 {
-	Widget scrollbar;
 	FileselectWidget w = (FileselectWidget) new;
 	Arg args[20];
 	register int n;
@@ -806,7 +831,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNborderWidth, (XtPointer) 0);			n++;
 	XtSetArg (args[n], XtNlabel, w->fileselect.label1);			n++;
 	w->fileselect.nameLabel1 =
-		XtCreateManagedWidget("nameLabel1", labelWidgetClass, w, args, n);
+		XtCreateManagedWidget("nameLabel1", labelWidgetClass,
+							  (Widget) w, args, n);
 
 	y_off = pad + w->fileselect.text_height + pad;
 
@@ -820,7 +846,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNborderWidth, (XtPointer) 0);			n++;
 	XtSetArg (args[n], XtNlabel, dirLabelMsg);					n++;
 	w->fileselect.dirLabel =
-		XtCreateManagedWidget("dirLabel", labelWidgetClass, w, args, n);
+		XtCreateManagedWidget("dirLabel", labelWidgetClass,
+							  (Widget) w, args, n);
 
 	n = 0;
 	XtSetArg (args[n], XtNresize, XawtextResizeWidth); n++;
@@ -834,7 +861,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNeditType, "edit");					n++;
 	XtSetArg (args[n], XtNstring, w->fileselect.directory);		n++;
 	w->fileselect.dirText =
-		XtCreateManagedWidget ("dirText", asciiTextWidgetClass, w, args, n);
+		XtCreateManagedWidget ("dirText", asciiTextWidgetClass,
+							   (Widget) w, args, n);
 
 	translations = XtParseTranslationTable (filter_translations);
 	XtOverrideTranslations (w->fileselect.dirText, translations);
@@ -851,7 +879,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNborderWidth, (XtPointer) 0);			n++;
 	XtSetArg (args[n], XtNlabel, filterLabelMsg);				n++;
 	w->fileselect.filterLabel =
-		XtCreateManagedWidget("filterLabel", labelWidgetClass, w, args, n);
+		XtCreateManagedWidget("filterLabel", labelWidgetClass,
+							  (Widget) w, args, n);
 
 	n = 0;
 	XtSetArg (args[n], XtNx, filterLabelMsgWidth + 2*pad);		n++;
@@ -865,7 +894,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNeditType, "edit");					n++; 
 	XtSetArg (args[n], XtNstring, w->fileselect.filter);		n++; 
 	w->fileselect.filterText =
-		XtCreateManagedWidget("filterText", asciiTextWidgetClass, w, args, n);
+		XtCreateManagedWidget("filterText", asciiTextWidgetClass,
+							  (Widget) w, args, n);
 
 	translations = XtParseTranslationTable (filter_translations);
 	XtOverrideTranslations (w->fileselect.filterText, translations);
@@ -888,7 +918,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNstring, get_new_filter_val (w));		n++; 
 	XtSetArg (args[n], XtNselectTypes, defaultSelectTypes);		 n++;
 	w->fileselect.selectText =
-		XtCreateManagedWidget ("selectText", asciiTextWidgetClass, w, args, n);
+		XtCreateManagedWidget ("selectText", asciiTextWidgetClass,
+							   (Widget) w, args, n);
 
 	translations = XtParseTranslationTable (select_translations);
 	XtOverrideTranslations (w->fileselect.selectText, translations);
@@ -903,14 +934,14 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 #define XtVaCMW XtVaCreateManagedWidget
 
 	w->fileselect.selectLabel =
-		XtVaCMW ("selectLabel", labelWidgetClass, w,
+		XtVaCMW ("selectLabel", labelWidgetClass, (Widget) w,
 				 XtNx, pad,
 				 XtNy, y_off,
 				 XtNborderWidth, (XtPointer) 0,
 				 XtNlabel, currFileLabelMsg,
 				 NULL);
 	w->fileselect.currSelectText =
-		XtVaCMW ("currSelectText", asciiTextWidgetClass, w,
+		XtVaCMW ("currSelectText", asciiTextWidgetClass, (Widget) w,
 				 XtNx, currFileLabelMsgWidth + 2*pad,
 				 XtNy, y_off,
 				 XtNwidth, MAX (10,
@@ -948,7 +979,8 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNwidth, w->core.width - 2*pad);		n++;
 	XtSetArg (args[n], XtNlabel, w->fileselect.label2);			n++;
 	w->fileselect.nameLabel2 =
-		XtCreateManagedWidget ("nameLabel2", labelWidgetClass, w, args, n);
+		XtCreateManagedWidget ("nameLabel2", labelWidgetClass,
+							   (Widget) w, args, n);
 
 
 	y_off += w->fileselect.text_height + pad;
@@ -962,20 +994,22 @@ select-end(PRIMARY,CUTBUFFER0)set_select()\n\
 	XtSetArg (args[n], XtNy, y_off);							n++;
 	XtSetArg (args[n], XtNlabel, "ok");						    n++;
 	w->fileselect.okButton =
-		XtCreateManagedWidget ("okButton", commandWidgetClass, w, args, n);
+		XtCreateManagedWidget ("okButton", commandWidgetClass,
+							   (Widget) w, args, n);
 
 	XtAddCallback (w->fileselect.okButton, XtNcallback, 
-				   do_fileselect, (XtPointer)NULL);
+				   (XtCallbackProc) do_fileselect, (XtPointer)NULL);
 
 	n = 0;
 	XtSetArg (args[n], XtNx, xtw (w, "	select	") + 2*pad);	n++;
 	XtSetArg (args[n], XtNy, y_off);							n++;
 	XtSetArg (args[n], XtNlabel, "cancel");						n++;
 	w->fileselect.cancelButton =
-		XtCreateManagedWidget ("cancelButton", commandWidgetClass, w, args, n);
+		XtCreateManagedWidget ("cancelButton", commandWidgetClass,
+							   (Widget) w, args, n);
 
 	XtAddCallback (w->fileselect.cancelButton, XtNcallback, 
-				   cancel_fileselect, (XtPointer)NULL);
+				   (XtCallbackProc) cancel_fileselect, (XtPointer)NULL);
 
 } /* Initialize */
 
@@ -992,7 +1026,6 @@ XEvent *event;
 Region region;
 {
 	FileselectWidget lw = (FileselectWidget) w;
-	GC gc;
 }
 
 
@@ -1126,7 +1159,7 @@ FileselectWidget w;
 				  xtw (w, "	 ok	 ") + 2*pad, y_off);
 
 	XtSetArg(args[0], XtNwidth, w->core.width);
-	XtSetValues(w, args, 1);
+	XtSetValues((Widget) w, args, 1);
 }
 
 /*
