@@ -7,7 +7,38 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: build.c,v $
+ * Revision 1.7  1992/12/09  04:57:09  moore
+ * in built makefile: when doing make install, use cp instead of mv.
+ * mv fails when doing a make on more than one machine, if the machines
+ * share file systems.
+ *
+ * XXX really should be smarter about this.
+ *
+ * Revision 1.6  1992/10/28  21:10:37  moore
+ * make "make" quieter by using '@' before invoking hairy shell stuff;
+ * add stuff for building makefiles for SGI.
+ *
+ * Revision 1.5  1992/08/05  21:53:02  moore
+ * - Add MAKE=make to generated Makefile.
+ * - Don't initialize ARCH or WHERE by default.  Make PVMMAKE macro work
+ *   if WHERE is not defined.
+ * - Fix generated Makefile to work correctly with pvmrsh code
+ *   (i.e. pass ARCH and HOME to inferior make when doing make WHERE=local)
+ *
+ * Revision 1.4  1992/06/06  03:56:55  moore
+ * fix generated makefile so that $(F77) is used if $(FC) is not defined
+ *
+ * Revision 1.3  1992/05/29  19:16:55  moore
+ * add code to generated makefiles to do "make everywhere" using pvmrsh.
+ * add toggle button to build panel to switch between "make here" and
+ * "make everywhere"
+ * rationalize (somewhat) handling of tmp files
+ * disable subdefs editing feature unless there is a subDefs resource
+ *
+ * Revision 1.2  1992/04/08  06:48:21  moore
+ * incorporates changes up to 8 April 1992
+ *
  *
  */
 
@@ -15,6 +46,7 @@ $Log$
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
+#include <pwd.h>
 #include "xincl.h"
 #include "rb.h"
 #include "param.h"
@@ -24,6 +56,11 @@ $Log$
 #include "xcomn.h"
 #include "comn.h"
 #include "funcdefs.h"
+#include "xbm/here.xbm"
+#include "xbm/everywhere.xbm"
+#ifdef index
+#undef index
+#endif
 
 #define SUBPROX 1				/* experimental subprocess stuff */
 #define BUTTONZ 1				/* experimental buttons stuff */
@@ -36,7 +73,9 @@ int build_ChildPid;
 /* #define	SUBDEFSFILE		"sub.defs" */
 #define	MAKEFILENAME	"Makefile"
 
+#ifndef errno
 extern int errno;				/* 4.3bsd needs this */
+#endif
 
 /*******************
 *  from elsewhere  *
@@ -77,10 +116,18 @@ static XtCallbackRec callback[2] = { { 0, 0 }, { 0, 0 } };
 static Arg args[16];
 static char crud[1024];
 
+/* name of temporary file to be nuked when a subproc completes */
+static char tmpFileName[128];
+
 #if 0
 static Widget makeButton;
 #endif
 
+static Widget whereToggleButton;
+static enum { local, everywhere } whereToggle = local;
+static Pixmap hereBitmap, everywhereBitmap;
+static void whereToggle_cb ();
+
 /*********************
  *                   *
  *  widget creation  *
@@ -129,9 +176,19 @@ Widget below;
 	Widget w2, w3;
 #endif
 	int n;
+	Window win = RootWindow (xDisp, xScrn);
 
 	if (buildCmd)
 		return buildCmd;
+
+	hereBitmap =
+		XCreatePixmapFromBitmapData (xDisp, win, here_bits,
+									 here_width, here_height,
+									 1, 0, 1);
+	everywhereBitmap =
+		XCreatePixmapFromBitmapData (xDisp, win, everywhere_bits,
+									 everywhere_width, everywhere_height,
+									 1, 0, 1);
 	n = 0;
 	XtSetArg(args[n], XtNfromVert, (XtArgVal)below); n++;
 	XtSetArg(args[n], XtNborderWidth, (XtArgVal)0); n++;
@@ -144,10 +201,14 @@ Widget below;
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
+	XtSetArg(args[n], XtNresizable, 1); n++;
 
 #ifdef BUTTONZ
-	w = cre_tog_but ("subdefs", buildCmd, args, n, (Widget)0, (Widget)0,
-					 "edit subdefs", (char*)0, 0, 0, subdefs_cb, 0, 0);
+	w = (Widget) NULL;
+	if (subDefsFile != NULL) {
+		w = cre_tog_but ("subdefs", buildCmd, args, n, (Widget)0, w,
+						 "edit subdefs", (char*)0, 0, 0, subdefs_cb, 0, 0);
+	}
 	w = cre_tog_but ("mkwraps", buildCmd, args, n, (Widget)0, w,
 					 "write wrappers", (char*)0, 0, 0, mkwraps_cb, 0, 0);
 	w = cre_tog_but ("mkmf", buildCmd, args, n, (Widget)0, w,
@@ -159,8 +220,11 @@ Widget below;
 	w = cre_tog_but ("mkinstall", buildCmd, args, n, (Widget) 0, w,
 					 "make install", (char*)0, 0, 0, make_install_cb, 0, 0);
 #else
-	w = cre_cmd_but ("subdefs", buildCmd, args, n, (Widget)0, (Widget)0,
-					 "edit subdefs", (char*)0, 0, 0, subdefs_cb, 0);
+	w = (Widget) NULL;
+	if (subDefsFile != NULL) {
+		w = cre_cmd_but ("subdefs", buildCmd, args, n, (Widget)0, w,
+						 "edit subdefs", (char*)0, 0, 0, subdefs_cb, 0);
+	}
 	w = cre_cmd_but ("mkwraps", buildCmd, args, n, (Widget)0, w,
 					 "write wrappers", (char*)0, 0, 0, mkwraps_cb, 0);
 	w = cre_cmd_but ("mkmf", buildCmd, args, n, (Widget)0, w,
@@ -171,7 +235,11 @@ Widget below;
 					 "make", (char*)0, 0, 0, make_cb, 0);
 	w = cre_cmd_but ("mkinstall", buildCmd, args, n, (Widget) 0, w,
 					 "make install", (char*)0, 0, 0, make_install_cb, 0);
-#endif				 
+#endif
+
+	whereToggleButton = cre_cmd_but ("whereToggle", buildCmd, args, n, 0, w,
+									 NULL, here_bits, here_width, here_height,
+									 whereToggle_cb, 0);
 	XtRealizeWidget(buildCmd);
 	return buildCmd;
 }
@@ -191,6 +259,7 @@ char *name;
 	char buf[5120];
 	int nread;
 	char *ptr1, *ptr2;
+	char *index ();
 
 	if ((nread = read (fd, buf, sizeof buf - 1)) <= 0)
 		return;
@@ -241,6 +310,13 @@ enum causeOfDeath howDied;
 	}
 
 	subproc_Pclose (pid);
+
+#if 0
+	if (*tmpFileName != '\0') {
+		unlink (tmpFileName);
+		*tmpFileName = '\0';
+	}
+#endif
 }
 
 /*
@@ -305,6 +381,9 @@ Widget w;
 XtPointer cli;
 XtPointer cd;
 {
+	if (subDefsFile == NULL)
+		return;
+
 	(void)sprintf (crud, editorCommand, subDefsFile, subDefsFile,
 				   subDefsFile);
 	build_RunChild ("editor", crud, w);
@@ -326,7 +405,6 @@ XtPointer cd;
  * really leave this out of SysV prior to R4?
  */
 
-#include <pwd.h>
 
 static char *
 getHome ()
@@ -347,6 +425,19 @@ getHome ()
 	return home;
 }
 
+static int
+WriteHosts ()
+{
+	FILE *fp;
+
+	sprintf (tmpFileName, "/tmp/htool.%d", getpid ());
+	if ((fp = fopen (tmpFileName, "w")) == NULL)
+		return EOF;
+	cm_WriteHosts (fp, costmatrix);
+	fclose (fp);
+	return 0;
+}
+
 static void
 make_cb (w, cli, cd)
 Widget w;
@@ -355,12 +446,13 @@ XtPointer cd;
 {
 	char *home = getHome ();
 
-#ifdef ARCHSTR
-	(void) sprintf (crud, "make ARCH=%s HOME=%s", ARCHSTR, home);
-#else
-	(void) sprintf (crud, "make HOME=%s", home);
-#endif
-
+	if (whereToggle == local)
+		(void) sprintf (crud, "make ARCH=%s HOME=%s", ARCHSTR, home);
+	else {
+		(void) WriteHosts ();
+		(void) sprintf (crud, "make ARCH=%s HOME=%s WHERE=%s",
+						ARCHSTR, home, tmpFileName);
+	}
 	build_RunChild ("make", crud, w);
 }
 
@@ -373,12 +465,13 @@ XtPointer cd;
 {
 	char *home = getHome ();
 
-#ifdef ARCHSTR
-	(void) sprintf (crud, "make clean ARCH=%s HOME=%s", ARCHSTR, home);
-#else
-	(void) sprintf (crud, "make clean HOME=%s", home);
-#endif
-
+	if (whereToggle == local)
+		(void) sprintf (crud, "make clean ARCH=%s HOME=%s", ARCHSTR, home);
+	else {
+		(void) WriteHosts ();
+		(void) sprintf (crud, "make clean ARCH=%s HOME=%s WHERE=%s",
+						ARCHSTR, home, tmpFileName);
+	}
 	build_RunChild ("make clean", crud, w);
 }
 
@@ -390,12 +483,13 @@ XtPointer cd;
 {
 	char *home = getHome ();
 
-#ifdef ARCHSTR
-	(void) sprintf (crud, "make install ARCH=%s HOME=%s", ARCHSTR, home);
-#else
-	(void) sprintf (crud, "make install HOME=%s", home);
-#endif
-
+	if (whereToggle == local)
+		(void) sprintf (crud, "make install ARCH=%s HOME=%s", ARCHSTR, home);
+	else {
+		(void) WriteHosts ();
+		(void) sprintf (crud, "make install ARCH=%s HOME=%s WHERE=%s",
+						ARCHSTR, home, tmpFileName);
+	}
 	build_RunChild ("make install", crud, w);
 }
 
@@ -405,7 +499,6 @@ Widget w;
 XtPointer cli;
 XtPointer cd;
 {
-	char tmpSubDefsFile[1024];
 	FILE *fp;
 
 	if (scratchgraph == NULL || gr_Empty (scratchgraph)) {
@@ -421,11 +514,9 @@ XtPointer cd;
 		return;
 	}
 
-	strcpy (tmpSubDefsFile, "/tmp/htoolXXXXXX");
-	mktemp (tmpSubDefsFile);
-	strcat (tmpSubDefsFile, ".s");
-	if ((fp = fopen (tmpSubDefsFile, "w")) == NULL) {
-		msg_Format ("can't create %s [%s]\n", tmpSubDefsFile, strerror(errno));
+	sprintf (tmpFileName, "/tmp/htool.%d", getpid ());
+	if ((fp = fopen (tmpFileName, "w")) == NULL) {
+		msg_Format ("can't create %s [%s]\n", tmpFileName, strerror(errno));
 		set_togg (w, 0);
 		return;
 	}
@@ -435,7 +526,7 @@ XtPointer cd;
 
 	sprintf (crud, "mkwrap %s %s",
 			 Language == LANG_FORTRAN ? "-f" : "-c",
-			 tmpSubDefsFile);
+			 tmpFileName);
 
 	build_RunChild ("mkwrap", crud, w);
 
@@ -455,6 +546,9 @@ Widget w;
 XtPointer cli;
 XtPointer cd;
 {
+	if (subDefsFile == NULL)
+		return;
+
 	/* XXX need to use a subprocess package instead of system () */
 	(void)sprintf (crud, editorCommand, subDefsFile, subDefsFile, subDefsFile);
 
@@ -477,11 +571,7 @@ XtPointer cd;
 	FILE *fp;
 	char *home = getHome ();
 
-#ifdef ARCHSTR
 	(void) sprintf (crud, "make ARCH=%s HOME=%s 2>&1", ARCHSTR, home);
-#else
-	(void) sprintf (crud, "make HOME=%s 2>&1", home);
-#endif
 
 	msg_Format ("running \"%s\"...\n", crud);
 	if ((fp = popen (crud, "r")) == NULL)
@@ -618,6 +708,10 @@ XtPointer cd;
 		"$(HLIBDIR)/librb.a",
 		"$(HLIBDIR)/libdl.a",
 		"$(HLIBDIR)/liballoc.a",
+#ifdef IMA_SGI
+		"/usr/lib/libsun.a",
+		"/usr/lib/libmalloc.a",
+#endif
 		"$(PLIB)"
 	};
 
@@ -655,21 +749,26 @@ XtPointer cd;
 	}
 
 #define FP fprintf
-#ifdef ARCHSTR
 	FP (ff, "# This variable needs to be set according to the architecture\n");
-	FP (ff, "# of the machine you are compiling on.  It is set by default\n");
-	FP (ff, "# to the architecture of the machine that generated the\n");
-	FP (ff, "# Makefile.  To compile a node program on another machine,\n");
-	FP (ff, "# either type \"make ARCH=something-else\"\n");
-	FP (ff, "# or just edit this Makefile to change the value below\n");
-	FP (ff, "ARCH\t=\t%s\n", ARCHSTR);
+	FP (ff, "# of the machine you are compiling on.  When htool runs make,\n");
+	FP (ff, "# it sets ARCH for you on the make command line.\n");
+	FP (ff, "# To compile a node program manually, you must\n");
+	FP (ff, "# supply ARCH yourself.  To do this, either type \n");
+	FP (ff, "# \"make ARCH=whatever\", or just edit this Makefile to\n");
+	FP (ff, "# explicitly set the value below.\n");
+	/*
+	 * Grrr.  Sun make does not let you override variables from the command
+	 * line.
+	 */
+	FP (ff, "#ARCH\t=\t%s\n", ARCHSTR);
 	FP (ff, "\n");
-#else
-	FP (ff, "# You need to change the value of ARCH to reflect the\n");
-	FP (ff, "# architecture name for your machine.\n");
-	FP (ff, "#ARCH\t=\t%s\n", "unknown");
+	FP (ff, "# The WHERE variable contains the name of a file that\n");
+	FP (ff, "# contains a list of hosts on which to compile this program.\n");
+	FP (ff, "# If $(WHERE) is \"local\" or empty, the stuff gets compiled\n");
+	FP (ff, "# on the local machine only.\n");
+	FP (ff, "# WHERE=local\n");
 	FP (ff, "\n");
-#endif
+
 	FP (ff, "# The following defines are set from these X resources:\n");
 	FP (ff, "# HLIBDIR\thtool.henceLibDir\n");
 	FP (ff, "# PLIB\thtool.pvmLib\n");
@@ -682,6 +781,7 @@ XtPointer cd;
 	FP (ff, "\n");
 	FP (ff, "# work around brain-damage in some versions of make\n");
 	FP (ff, "SHELL\t=\t/bin/sh\n");
+	FP (ff, "MAKE\t=\tmake\n");
 	FP (ff, "\n");
 
 #define SIZE(x) (sizeof (x) / sizeof (*x))
@@ -699,11 +799,11 @@ XtPointer cd;
 	}
 
     FP (ff, "\n");
-	FP (ff, "# These are commented out because the correct definitions\n");
+	FP (ff, "# The correct definitions of these macros\n");
 	FP (ff, "# vary from one system to another.\n");
-	FP (ff, "# Uncomment and redefine as necessary.\n");
+	FP (ff, "# Uncomment and/or redefine as necessary.\n");
 	FP (ff, "#CC\t=\tcc\n");
-	FP (ff, "#FC\t=\tf77\n");
+	FP (ff, "FORT\t=\t`case \"$(FC)@$(F77)\" in *@) echo $(FC) ;; @*) echo $(F77) ;; *) echo f77;; esac`\n");
 	FP (ff, "#\n");
 	FP (ff, "# CFLAGS is used here to allow us to add the -DIMA_$(ARCH)\n");
 	FP (ff, "# option to the default compile rule for C programs.\n");
@@ -713,32 +813,47 @@ XtPointer cd;
 	FP (ff, "#CDEBUGFLAGS=-g\n");
 	FP (ff, "CFLAGS\t=\t-DIMA_$(ARCH) $(CDEBUGFLAGS)\n");
     FP (ff, "\n");
-	FP (ff, "all:");
+	FP (ff, "PROGRAMS=");
 	for (tn = rb_First(st); tn != st; tn = rb_Next(tn))
 		if (strcmp (st_Key(tn), "null") != 0)
-			fprintf(ff, " %s", st_Key(tn));
-    FP (ff, "\n");
+			fprintf(ff, "%s ", st_Key(tn));
     FP (ff, "\n");
 
-	FP (ff, "install: all\n");
+	FP (ff, "PVMMAKE=set -x ; case \"$(WHERE)\" in \\\n");
+	FP (ff, "\tlocal|\"\") \\\n");
+	FP (ff, "\t\t$(MAKE) $(MFLAGS) ARCH=$(ARCH) HOME=$(HOME) $@.local ;;\\\n");
+	FP (ff, "\t*)pvmrsh -h `cat $(WHERE)` -f *.[chf] Makefile -c \\\n");
+	FP (ff,
+		"\t\tmake $(MFLAGS) $@.local 'HOME=$$(HOME)' 'ARCH=$$(ARCH)' ;;\\\n");
+	FP (ff, "\tesac\n");
+	FP (ff, "\n");
+
+	FP (ff, "all:\n");
+	FP (ff, "\t@$(PVMMAKE)\n");
+    FP (ff, "\n");
+
+	FP (ff, "all.local: $(PROGRAMS)\n");
+	FP (ff, "\n");
+
+	FP (ff, "install:\n");
+	FP (ff, "\t@$(PVMMAKE)\n");
+    FP (ff, "\n");
+
+	FP (ff, "install.local: $(PROGRAMS)\n");
 	FP (ff, "\t@- if [ ! -d $(XDIR) ] ; then \\\n");
 	FP (ff, "\t\techo creating dir $(XDIR) ; \\\n");
 	FP (ff, "\t\tmkdir $(XDIR) ; \\\n");
 	FP (ff, "\tfi ; \\\n");
 	FP (ff, "\texit 0\n");
-	FP (ff, "\tmv");
-	for (tn = rb_First(st); tn != st; tn = rb_Next(tn))
-		if (strcmp (st_Key(tn), "null") != 0)
-			fprintf(ff, " %s", st_Key(tn));
-	FP (ff, " $(XDIR)\n");
+	FP (ff, "\tcp $(PROGRAMS) $(XDIR)\n");
 	FP (ff, "\n");
 
 	FP (ff, "clean:\n");
-	FP (ff, "\trm -f");
-	for (tn = rb_First(st); tn != st; tn = rb_Next(tn))
-		if (strcmp (st_Key(tn), "null") != 0)
-			fprintf(ff, " %s", st_Key(tn));
-	FP (ff, " *.o\n");
+	FP (ff, "\t@$(PVMMAKE)\n");
+	FP (ff, "\n");
+
+	FP (ff, "clean.local:\n");
+	FP (ff, "\trm -f $(PROGRAMS) *.o\n");
 	FP (ff, "\n");
 
 	if (Language == LANG_FORTRAN) {
@@ -747,8 +862,8 @@ XtPointer cd;
 				continue;
 			FP (ff, "%s:	%s.o w_%s.o $(FMAIN) $(HLIBS)\n",
 				st_Key (tn), st_Key (tn), st_Key (tn));
-			FP (ff,
-				"\t$(FC) $(FFLAGS) -o %s %s.o w_%s.o $(HLIBS) $(FMAIN)\n\n",
+			FP (ff, "\t@FC=\"$(FORT)\" ; set -x;\\\n");
+			FP (ff, "\t$$FC $(FFLAGS) -o %s %s.o w_%s.o $(HLIBS) $(FMAIN)\n\n",
 				st_Key (tn), st_Key (tn), st_Key (tn));
 			FP (ff, "w_%s.o:\tfw_%s.c\n", st_Key (tn), st_Key (tn));
 			FP (ff, "\t$(CC) $(CFLAGS) $(INCLUDES)  -c fw_%s.c\n",
@@ -789,6 +904,23 @@ XtPointer cd;
 	return;
 }
 
+static void
+whereToggle_cb ()
+{
+	if (whereToggle == local)
+		whereToggle = everywhere;
+    else
+		whereToggle = local;
+
+	XtVaSetValues (whereToggleButton,
+				   XtNbitmap,
+				   whereToggle == local ? hereBitmap : everywhereBitmap,
+				   NULL);
+	return;
+}
+
+
+
 /*
  * Local variables:
  * tab-width:4
