@@ -7,7 +7,10 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: comn.h,v $
+ * Revision 1.1  1992/04/08  05:48:26  moore
+ * initial RCS version
+ *
  *
  */
 
