@@ -8,7 +8,22 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: compose.c,v $
+ * Revision 1.5  1992/12/09  04:58:10  moore
+ * add casts to make alpha compiler happy.
+ *
+ * Revision 1.4  1992/09/18  01:42:59  moore
+ * improve error message when an attempt to parse a node program fails.
+ *
+ * Revision 1.3  1992/06/23  19:34:40  moore
+ * put node # on "no errors" msg after parsing a newly-edited node.
+ *
+ * Revision 1.2  1992/05/29  19:18:25  moore
+ * get rid of subdefs stuff unless there is a subDefs resource
+ *
+ * Revision 1.1  1992/04/08  05:48:30  moore
+ * initial RCS version
+ *
  *
  */
 
@@ -319,8 +334,11 @@ cre_cmdpanel(below)
 		(Widget)0, w, "clear", (char*)0, 0, 0, clear_cb, 0);
 	w = cre_cmd_but("critic", composeCmd, args, n,
 		(Widget)0, w, "critic", (char*)0, 0, 0, critic_cb, 0);
-	w = cre_cmd_but("subdefs", composeCmd, args, n,
-		(Widget)0, w, "edit subdefs", (char*)0, 0, 0, subdefs_cb, 0);
+	if (subDefsFile != NULL) {
+		w = cre_cmd_but("subdefs", composeCmd, args, n,
+						(Widget)0, w, "edit subdefs", (char*)0, 0, 0,
+						subdefs_cb, 0);
+	}
 	w = cre_cmd_but("cleanup", composeCmd, args, n,
 		(Widget)0, w, "cleanup", (char*)0, 0, 0, cleanup_cb, 0);
 	w = cre_cmd_but("redraw", composeCmd, args, n,
@@ -944,8 +962,10 @@ compose_help_cb(w, cli, cd)
 
     if (helpVisible = !helpVisible) {
         if (!helpPop) {
-            helpPop = XtCreatePopupShell("Compose Help", topLevelShellWidgetClass,
-                topLevel, NULL, NULL);
+            helpPop = XtCreatePopupShell ("Compose Help",
+										  topLevelShellWidgetClass,
+										  topLevel,
+										  (ArgList) NULL, 0);
 
             n = 0;
             XtSetArg(args[n], XtNheight, compose_help_height); n++;
@@ -1003,9 +1023,10 @@ subdefs_cb (w, cli, cd)
 Widget w;
 XtPointer cli, cd;
 {
-	void compose_EditNodeSourceProgram ();
+	static void compose_EditNodeSourceProgram ();
 
-	compose_EditNodeSourceProgram (subDefsFile);
+	if (subDefsFile != NULL)
+		compose_EditNodeSourceProgram (subDefsFile);
 }
 
 /*	node_t_cb()
@@ -1542,7 +1563,7 @@ struct nodeprogram *p;
 			 * The new node program parses without errors.  Replace
 			 * the old node in the graph with the new one.
 			 */
-			msg_Format ("no errors.");
+			msg_Format ("Node %d: no errors.", newnode->nk.id);
 			xgr_DrawNode (graphWin, p->node, 0);
 			gr_PatchNode (p->node, newnode);
 			gr_FreeNode (newnode);
@@ -1556,10 +1577,14 @@ struct nodeprogram *p;
 			 * parser detected node program errors.  Offer to edit
 			 * again, otherwise throw away this node.
 			 */
+			char msgbuf[100];
+
+			sprintf (msgbuf, "Error in program for node %d.  Edit again?",
+					 p->node->nk.id);
 			xgr_DrawNode (graphWin, p->node, 0);
 			p->node->state = ST_WARNING;
 			xgr_DrawNode (graphWin, p->node, 2);
-			if (verify ("Error in node program. Edit again?") == 0) {
+			if (verify (msgbuf) == 0) {
 				compose_refresh ();
 				sprintf (crud, editorCommand, p->filename, p->filename,
 						 p->filename);
