@@ -5,6 +5,8 @@
  */
 
 #include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "comn.h"
 #include "xincl.h"
 #include "xcomn.h"
@@ -781,7 +783,6 @@ hmap_ResetHosts()
 
     for (tn = rb_First (hosts); tn != hosts; tn = rb_Next (tn)) {
         struct host_info *h = (struct host_info *) rb_Value (tn);
-		TreeNode tn2, tn3;
 
         if (h->name)
             h->numIdle = h->numRunning = 0;
@@ -794,6 +795,31 @@ hmap_ResetHosts()
 	/* 	draw_hosts(hostsWin); */
 }
 
+char **hmap_HostList = NULL;
+int hmap_NumHosts = 0;
+
+void
+hmap_MakeHostList ()
+{
+	TreeNode tn;
+	int i;
+	
+	hmap_HostList = NULL;
+	hmap_NumHosts = hostCount;
+
+	if (hostCount <= 0)
+		return;
+	if (hosts == (Tree) NULL || rb_Empty (hosts))
+		return;
+
+	hmap_HostList = (char **) malloc (hostCount * sizeof (char *));
+	for (i = 0, tn = rb_First (hosts) ; tn != hosts ; tn = rb_Next (tn)) {
+		struct host_info *h = (struct host_info *) rb_Value (tn);
+		h->hostNumber = i;
+		hmap_HostList[i++] = h->name;
+	}
+}
+
 /*
  * Local variables:
  * tab-width:4
