@@ -23,6 +23,7 @@ struct icon {
 struct host_info {
     char *name;			/* name of the host */
     char string[255];		/* string to go under host name */
+    int hostNumber;		/* relative host number (for histogram) */
     int numRunning;		/* number of host processes running */
     int numIdle;		/* number of host processes idle */
     int xpos;			/* x position on window */
@@ -49,4 +50,7 @@ void hmap_HostEvent __((struct host_info *h, int id, int inst,
 			int oldstate, int newstate));
 void hmap_ResetHosts __(());
 void hmap_NukeHosts __(());
+void hmap_MakeHostList __(());
+extern char **hmap_HostList;
+extern int hmap_NumHosts;
 #undef __
