@@ -7,7 +7,10 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: symtab.c,v $
+ * Revision 1.1  1992/04/08  05:49:47  moore
+ * initial RCS version
+ *
  *
  */
 
