@@ -8,7 +8,38 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: trace.c,v $
+ * Revision 1.9  1992/12/09  05:06:12  moore
+ * nuke gratuitous printfs.
+ * tweak some type declarations, add some casts, to make alpha happy.
+ *
+ * Revision 1.8  1992/09/18  01:53:13  moore
+ * add label widget displaying elapsed time or current trace event
+ * trace is now time-scaled.
+ * 'elapsed' is now 'print_time"
+ * 'elapsed_time' and 'after' copied from histo.c; need to move them
+ *     elsewhere at some future date.
+ * changed 'serial' to 'millisecs' in trace events
+ * 'usageIsVisible' changed to 'histogramExists'
+ *
+ * Revision 1.7  1992/08/24  22:05:57  moore
+ * take out debugging printfs
+ *
+ * Revision 1.6  1992/08/05  21:59:23  moore
+ * made mods to handle fractional (msec) time.  Uses new trace file format.
+ *
+ * Revision 1.5  1992/07/10  21:56:03  moore
+ * hacked-in support for histogram window
+ *
+ * Revision 1.4  1992/07/10  21:53:23  moore
+ * fix nasty memory trashing bug in elapsed()
+ *
+ * Revision 1.3  1992/05/19  05:39:16  moore
+ * add casts to make MetaWare hc happy.
+ *
+ * Revision 1.2  1992/04/08  06:50:30  moore
+ * incorporates changes up to 8 April 1992
+ *
  *
  */
 
@@ -19,7 +50,11 @@ $Log$
  * - trace in either direction
  */
 
+#define USAGE_GRAPH
+#define TIME_LABEL
+#define TIME_SCALE
 
+#include <math.h>
 #include <sys/types.h>
 #include <stdio.h>
 #include <signal.h>
@@ -35,6 +70,7 @@ $Log$
 #include "costmat.h"
 #include "hostmap.h"
 #include "tf.h"
+#include "histo.h"
 
 #include "xbm/legend.xbm"
 #include "xbm/fwd.xbm"
@@ -47,7 +83,9 @@ $Log$
 #include "xbm/usage.xbm"
 #endif
 
+#ifndef errno
 extern int errno;				/* 4.3 BSD needs this */
+#endif
 
 /*******************
 *  from elsewhere  *
@@ -69,6 +107,7 @@ void start_trace();
 int stop_trace();
 Widget hostsWin = NULL;			/* used by hostmap.c */
 Widget hostsPop = NULL;			/* used by hostmap.c */
+Widget usagePop = NULL;
 
 /***********
 *  global  *
@@ -97,17 +136,27 @@ static Widget rewindButton = 0;
 static Widget stopButton = 0;
 static Widget fwdButton = 0;
 static Widget fwdstepButton = 0;
+#ifdef TIME_LABEL
+static Widget timeLabel = 0;
+#endif
 
 static Arg args[16];
 #if 0
 static char crud[1024];
 #endif
 static Window graphWin;
+#ifdef TIME_SCALE
+static float time_scale = 1.0;
+#define TIMER_ALARM 0
+#define READ_AGAIN 1
+#define FIRST_EVENT 2
+#else
 static int animat_delay = 0;
+#endif
 static XtCallbackRec callback[2] = { { 0, 0 }, { 0, 0 } };
 static Graph tracegraph = 0;
 static int monitorMode = 0;
-
+Boolean histogramExists = False;
 
 
 /*********************
@@ -123,6 +172,8 @@ static int monitorMode = 0;
  *	Set default graph file name from main module.
  */
 
+static int checkThings ();
+
 void
 start_trace()
 {
@@ -243,6 +294,7 @@ cre_cmdpanel(below)
 	XtSetArg(args[n], XtNfromVert, (XtArgVal)below); n++;
 	XtSetArg(args[n], XtNborderWidth, (XtArgVal)0); n++;
 	XtSetArg(args[n], XtNhorizDistance, (XtArgVal)0); n++;
+	XtSetArg(args[n], XtNresizable, (XtArgVal)True); n++;
 	retw = traceCmd = XtCreateWidget("traceCmd", formWidgetClass,
 			mainForm, args, n);
 
@@ -290,7 +342,11 @@ cre_cmdpanel(below)
 	XtSetArg(args[n], XtNwidth, 200); n++;
 	XtSetArg(args[n], XtNheight, 20); n++;
 	{
+#ifdef TIME_SCALE
+		float f = 0.5;
+#else
 		float f = 1.0;
+#endif
 
 		if (sizeof(f) > sizeof(args[0].value)) {
 			XtSetArg(args[n], XtNtopOfThumb, &f); n++;
@@ -307,7 +363,7 @@ cre_cmdpanel(below)
 	w = speedKnob = XtCreateManagedWidget("execSpeed",
 		scrollbarWidgetClass, traceCmd, args, n);
 	n = saved_n;
-	/* SPEED SLIDER */
+	/* END SPEED SLIDER */
 
 #if 0
 	/*
@@ -333,7 +389,20 @@ cre_cmdpanel(below)
 					 usage_bits, usage_width, usage_height, usage_cb, 0, 0);
 #endif /* USAGE_GRAPH */
 
-
+#ifdef TIME_LABEL
+	timeLabel = XtVaCreateManagedWidget ("timeLabel", labelWidgetClass,
+										 traceCmd,
+										 XtNleft, XtChainLeft,
+										 XtNright, XtChainRight,
+										 XtNtop, XtChainTop,
+										 XtNbottom, XtChainBottom,
+										 XtNresizable, False,
+										 XtNlabel, "00:00:00.000",
+										 XtNheight, 20,
+										 XtNfromHoriz, (XtArgVal) w,
+										 NULL);
+	w = timeLabel;
+#endif
 
 	XtRealizeWidget(traceCmd);
 	return traceCmd;
@@ -446,7 +515,9 @@ stopTrace ()
 	extern Widget grfButton;	/* in widmain.c */
 
 	if (!traceStopped) {
+#ifndef TIME_SCALE
 		XtRemoveTimeOut (timerId);
+#endif
 		traceStopped = 1;
 #if 1
 		XtVaSetValues (traceFileButton, XtNsensitive, True, NULL);
@@ -456,23 +527,123 @@ stopTrace ()
 }
 
 
+static void Tick ();
+
 static void
 startTrace ()
 {
 	extern Widget traceFileButton; /* in widmain.c */
 	extern Widget grfButton;	/* in widmain.c */
-	void Tick ();
 
 	if (traceStopped) {
+#ifdef TIME_SCALE
+		/*
+		 * do the first tick manually.  After that, Tick() will schedule
+		 * the next tick according to the value of time_scale.
+		 */
+		traceStopped = 0;
+		XtVaSetValues (traceFileButton, XtNsensitive, False, NULL);
+		XtVaSetValues (grfButton, XtNsensitive, False, NULL);
+		Tick ((XtPointer) FIRST_EVENT, 0);
+#else
 		timerId = XtAppAddTimeOut (context, animat_delay, Tick, NULL);
 		traceStopped = 0;
-#if 1
 		XtVaSetValues (traceFileButton, XtNsensitive, False, NULL);
 		XtVaSetValues (grfButton, XtNsensitive, False, NULL);
 #endif
 	}
 }
 
+static char *
+print_time (t)
+struct EventTime *t;
+{
+	static char buf[30];
+	char *ptr;
+	int msec = t->millisecs;
+	long secs = t->secs;
+
+#if 0
+	sprintf (buf, "%0d.%03d",
+			 secs, msec);
+	return buf;
+#else
+	ptr = buf + sizeof(buf);
+	
+	*--ptr = '\0';
+	*--ptr = msec % 10 + '0';
+	msec /= 10;
+	*--ptr = msec % 10 + '0';
+	msec /= 10;
+	*--ptr = msec % 10 + '0';
+	msec /= 10;
+	*--ptr = '.';
+	*--ptr = (secs % 10) + '0';
+	secs /= 10;
+	*--ptr = (secs % 6) + '0';
+	secs /= 6;
+	*--ptr = ':';
+	*--ptr = (secs % 10) + '0';
+	secs /= 10;
+	*--ptr = (secs % 6) + '0';
+	secs /= 6;
+	*--ptr = ':';
+	*--ptr = (secs % 10) + '0';
+	secs /= 10;
+	*--ptr = (secs % 10) + '0';
+	while ((secs /= 10) > 0)
+		*--ptr = (secs % 10) + '0';
+		   
+	return ptr;
+#endif
+}
+
+
+/*
+ * XXX this is copied from histo.c.  Eventually put all of the event/
+ * time management stuff in a separate file.
+ */
+
+static struct EventTime *
+elapsed_time (t1, t2)
+struct EventTime *t1, *t2;
+{
+    static struct EventTime result;
+
+	result.secs = 0;
+    result.millisecs = t2->millisecs - t1->millisecs;
+    if (result.millisecs < 0) {
+		result.millisecs += 1000;
+		result.secs = -1;
+    }
+    result.secs = result.secs + t2->secs - t1->secs;
+#ifdef DEBUG
+	if (result.millisecs < 0) {
+		fprintf (stderr, "elapsed_time (%d.%03d %d.%03d) -> %d.%03d\n",
+				 t1->secs, t1->millisecs,
+				 t2->secs, t2->millisecs,
+				 result.secs, result.millisecs);
+	}
+#endif
+    return &result;
+}
+
+/*
+ * XXX also stolen from histo.c
+ */
+
+static int 
+after (t1, t2)
+struct EventTime *t1, *t2;
+{
+    if (t1->secs > t2->secs)
+		return 1;
+    else if (t1->secs < t2->secs)
+		return 0;
+    else
+		return (t1->millisecs > t2->millisecs);
+}
+
 /*
  * Come here to process a trace event.
  *
@@ -482,11 +653,15 @@ startTrace ()
  *
  */
 
+static struct EventTime epoch;
+
 static void
 traceEvent (tev)
 struct trace_event *tev;
 {
 	Node n;
+	struct EventTime now;
+	struct EventTime *delta;
 
 	if (tev->id == -1 && tev->inst == -1)
 		;
@@ -496,13 +671,29 @@ struct trace_event *tev;
 			return;
 	}
 
-#if 1
-	msg_Format ("%ld %ld %d/%d %s %c %d %d\n",
-				tev->timestamp.secs, tev->timestamp.serial,
+	if (tev->event_type == TEV_START) {
+		epoch.secs = tev->timestamp.secs;
+		epoch.millisecs = tev->timestamp.millisecs;
+	}
+	now.secs = tev->timestamp.secs;
+	now.millisecs = tev->timestamp.millisecs;
+		
+	delta = elapsed_time (&epoch, &now);
+	msg_Format ("%s %d/%d %s %c %d %d\n",
+				print_time (delta),
 				tev->id, tev->inst,
 				tf_GetEventName (tev->event_type),
 				tev->tf ? 'T' : 'F',
 				tev->lo, tev->hi);
+
+#ifdef TIME_LABEL
+	set_label (timeLabel, print_time (delta));
+#endif
+	if (monitorMode == 1 && histogramExists)
+		hist_SetEndOfTime ((struct EventTime *) &(tev->timestamp));
+#ifdef HAIRLINE
+	if (histogramExists)
+		hist_SetHairLine ((struct EventTime *) &(tev->timestamp), 1);
 #endif
 
 	switch (tev->event_type) {
@@ -672,6 +863,145 @@ struct trace_event *tev;
  * tracing, read another event and deal with it.
  */
 
+#ifdef TIME_SCALE
+static void
+Tick (mode, y)
+XtPointer mode;
+XtIntervalId *y;
+{
+	static struct trace_event thisEvent;	/* this event */
+	static struct EventTime thisEventTime;	/* time of this event */
+	static struct EventTime pseudoNow;		/* simulated time */
+
+	switch ((int) mode) {
+	case TIMER_ALARM:			/* timer went off */
+		/*
+		 * NB: when the timer goes off, we don't check traceStopped
+		 * until we've processed at least one event.  This is because
+		 * we've already read an event into thisEvent, but we haven't
+		 * yet processed it.
+		 */
+		break;
+	case READ_AGAIN:			/* file read failed last time, retry */
+		if (traceStopped)
+			return;
+		if (tf_NextEvent (&thisEvent) != 0)
+			goto readfail;
+		thisEventTime.secs = thisEvent.timestamp.secs;
+		thisEventTime.millisecs = thisEvent.timestamp.millisecs;
+		break;
+	case FIRST_EVENT:			/* called from startTrace to prime things */
+		if (tf_NextEvent (&thisEvent) != 0)
+			goto readfail;
+		thisEventTime.millisecs = thisEvent.timestamp.millisecs;
+		thisEventTime.secs = thisEvent.timestamp.secs;
+		pseudoNow = thisEventTime;
+		break;
+	}
+
+	/*
+	 * handle all events up to and including the trace time
+	 */
+	do {
+		struct EventTime *delta;
+		long delay_time;
+		long msec;
+
+		/*
+		 * if it's too early to trace this event, update hairline
+		 * and elapsed time indicator, and wait a while.
+		 */
+		if (after (&thisEventTime, &pseudoNow)) {
+#ifdef TIME_LABEL
+			delta = elapsed_time (&epoch, &pseudoNow);
+			set_label (timeLabel, print_time (delta));
+#endif /* TIME_LABEL */
+			if (histogramExists)
+				hist_SetHairLine (&pseudoNow, 1);
+
+			/*
+			 * calculate time to delay until next event is traced.
+			 * if the histogram window is visible, this wants to
+			 * be the amount of scaled time equal to a pixel; 
+			 * otherwise, this should be the scaled time until another 
+			 * event takes place.
+			 */
+			if (histogramExists) {
+				if (hist_MsecPerPixel (&msec) < 0)
+					msec = 1000;
+			}
+			else {
+				delta = elapsed_time (&pseudoNow, &thisEventTime);
+				msec = delta->secs * 1000 + delta->millisecs;
+			}
+			delay_time = msec * time_scale;
+
+			/*
+			 * if delay_time is longer than one second, reset msec and
+			 * delay_time so that we will wake up within a second.
+			 * This is so the "stop" button will work within a
+			 * reasonable time.  If possible, make the clock stop on an
+			 * even second boundary.  This is because it looks wierd
+			 * to watch the middle digits of the clock tick by but always
+			 * with the same lower order digits.
+			 */
+			if (delay_time > 1000) {
+				delta = elapsed_time (&epoch, &pseudoNow);
+				msec = 1000 - delta->millisecs;
+				if (msec == 0)
+					msec = 1000;
+				delay_time = msec * time_scale;
+				if (delay_time > 1000) {
+					msec = 1000 / time_scale;
+					delay_time = msec * time_scale;
+				}
+			}
+			/*
+			 * on the other hand, if delay_time is too short (< 10msec),
+			 * increase the amount of clock time to be elapsed until
+			 * delay_time is long enough for a minimum delay.
+			 */
+			else if (delay_time < 10) {
+				msec = (int) ((10.0 / time_scale) + 0.5);
+				delay_time = msec * time_scale;
+			}
+
+			/*
+			 * figure out what time it will be when we wake up.
+			 */
+			pseudoNow.millisecs += msec;
+			pseudoNow.secs += pseudoNow.millisecs / 1000;
+			pseudoNow.millisecs = pseudoNow.millisecs % 1000;
+
+			/*
+			 * set the timer to go off then.
+			 */
+			timerId = XtAppAddTimeOut (context, delay_time, Tick,
+									   (XtPointer) TIMER_ALARM);
+			return;
+		}
+		traceEvent (&thisEvent);
+		if (traceStopped)
+			return;
+		if (tf_NextEvent (&thisEvent) != 0)
+			goto readfail;
+		thisEventTime.secs = thisEvent.timestamp.secs;
+		thisEventTime.millisecs = thisEvent.timestamp.millisecs;
+	} while (1);
+
+readfail:
+	if (monitorMode) {
+		/* wait 500 msec and try again */
+		timerId = XtAppAddTimeOut (context, 500, Tick, (XtPointer) READ_AGAIN);
+	}
+	else {
+		msg_Format ("%s: premature end of file\n", traceFileName);
+		stopTrace ();
+		set_togg (fwdButton, 0);
+		set_togg (stopButton, 1);
+	}
+}
+#else
 static void
 Tick (x, y)
 XtPointer x;
@@ -714,6 +1044,7 @@ XtIntervalId *y;
 	/* reset time out */
 	timerId = XtAppAddTimeOut (context, animat_delay, Tick, x);
 }
+#endif
 
 #if 0
 /*	redraw_cb()
@@ -807,7 +1138,8 @@ XtPointer cd;
 	set_togg (stopButton, 1);
 }
 
-/*	rewind_cb()
+/*
+ *	rewind_cb()
  *
  *	Rewind command
  */
@@ -832,9 +1164,16 @@ XtPointer cd;
 	 */
 	gr_ResetGraph (tracegraph, ST_NOT_BEGUN);
 	trace_refresh ();
+#ifdef HAIRLINE
+	if (histogramExists)
+		hist_ClearHairLine ();
+#endif
 
 	set_togg (rewindButton, 0);
 	set_togg (stopButton, 1);
+#ifdef TIME_LABEL
+	set_label (timeLabel, "00:00:00.000");
+#endif
 	msg_Format ("Rewound trace file.\n");
 }
 
@@ -852,7 +1191,22 @@ XtPointer cd;
 	float f;
 
 	f = *(float*)cd;
-	animat_delay = (1.0 - f) * 2000;
+#ifdef TIME_SCALE
+	/*
+	 * need a log scale
+	 * if slider is at the far left, time_scale = 100
+	 * if slider is in the center, time_scale = 1.0
+	 * if slider is at the far right, time_scale = .01
+	 * total magnitude difference = 10 ** 4;
+	 */
+
+	time_scale = pow (10.0, (1.0 - f) * 4.0 - 2.0);
+#ifdef DEBUG
+	fprintf (stderr, "time scale = %f\n", time_scale);
+#endif
+#else
+	animat_delay = (1.0 - f) * 2;
+#endif
 }
 
 
@@ -887,7 +1241,7 @@ XtPointer cd;
 {
 	Arg args[20];
 	register int n;
-	int x, y, curr_x, curr_y;
+	Position x, y, curr_x, curr_y;
 	Widget form;
 #if 0
 	Widget button;
@@ -919,7 +1273,8 @@ XtPointer cd;
 
 		curr_x = curr_y = 400;
 
-		XtTranslateCoords (w, curr_x, curr_y, &x, &y);
+		XtTranslateCoords (w, curr_x, curr_y,
+						   (Position *) &x, (Position *) &y);
 
 		n = 0;
 		XtSetArg(args[n], XtNx, x);                         n++;
@@ -955,12 +1310,32 @@ XtPointer cd;
 }
 
 #ifdef USAGE_GRAPH
+static void ScanTraceFile ();
+
 static void
 usage_cb (w, cli, cd)
 Widget w;
 XtPointer cli;
 XtPointer cd;
 {
+	if (histogramExists == 0) {
+		if (checkThings () != 0) {
+			set_togg (w, 0);
+			return;
+		}
+		usagePop = XtAppCreateShell ("Utilization graph", "UsagePopup",
+									 applicationShellWidgetClass,
+									 XtDisplay (w),
+									 NULL, 0);
+		setUpHistogram ();
+		XtPopup (usagePop, XtGrabNone);
+		histogramExists = 1;
+	}
+	else {
+		XtDestroyWidget (usagePop);
+		usagePop = NULL;
+		histogramExists = 0;
+	}
 }
 #endif
 
@@ -1018,180 +1393,329 @@ trace_refresh()
 	xgr_DrawGrf(graphWin, tracegraph, 1);
 }
 
-#if 0
-/**************************
-*  trace animation stuff  *
-*                         *
-**************************/
+#ifdef USAGE_GRAPH
+#ifdef FAKE_DATA
+struct Event lastEventRead;
 
-
-read_tracefile()
+int
+getNextEvent (nextEvent)
+struct Event *nextEvent;
 {
-	FILE *ff;
-	char buf[256];
-	struct tracev tev;
-	char nodetype[32];
-	char machine[128];
-	char evtypnm[128];
-	char tf;
-	int i;
-	char **pp;
+    int hostNum = lastEventRead.hostno;
+    int curTime = lastEventRead.time;
+    int oldHostState, newHostState;
 
-	if ((ff = fopen(traceFileName, "r")) == NULL) {
-		msg_Format ("can't open trace file \"%s\"\n", traceFileName);
-		return;
-	}
-	while (xfgets(buf, sizeof(buf), ff)) {
-		if (buf[0] != '#')
-			continue;
-		if (sscanf(buf + 1, "%ld %d/%d %s %s %s %c %d %d",
-				&tev.time, &tev.id, &tev.inst, nodetype, machine,
-				evtypnm, &tf, &tev.lo, &tev.hi) != 9)
-			continue;
-		for (pp = evtypenames; *pp && strcmp(evtypnm, *pp); pp++);
-		tev.evtype = pp - evtypenames;
-		tev.tf = tf == 'T';
+#define HOST_STATE(host,time) ((((time)%(2*(host+1)+2)) / (host+2) == 0) + 1)
 
-		trace_event(machine, tracegraph, &tev);
-		checkevents();
-		if (whoa)
-			break;
-		delay (animat_delay);
-	}
-	fclose(ff);
+    do {
+		++hostNum;
+		if (hostNum >= hmap_NumHosts) {
+			hostNum = 0;
+			curTime++;
+		}
+		oldHostState = HOST_STATE(hostNum,curTime-1);
+		newHostState = HOST_STATE(hostNum,curTime);
+    } while (oldHostState == newHostState);
+
+    nextEvent->hostno = hostNum;
+    nextEvent->time = curTime;
+    nextEvent->state = newHostState;
+    lastEventRead = *nextEvent;
+    return 0;
 }
 
-trace_event(machine, grf, tvp)
-char *machine;
-Graph grf;
-struct tracev *tvp;
+void
+getState (stateArray, numHosts, timep)
+struct Event *stateArray;
+int numHosts;
+long *timep;
 {
-	Node n;
+    int i;
 
-	if (tvp->id == -1 && tvp->inst == -1)
-		;
-	else if ((n = gr_GetNodeByIdInst (grf, tvp->id, tvp->inst)) == (Node) NULL)
+
+    for (i = 0; i < hmap_NumHosts; ++i) {
+		stateArray[i].state = HOST_STATE(i, *timep - 1);
+		stateArray[i].hostno = i;
+		stateArray[i].time = *timep - 1;
+		stateArray[i].fractime = 0;
+    }
+    lastEventRead = stateArray[hmap_NumHosts-1];
+    return 0;
+}
+#else
+
+static char *hostNames[200];
+static int numRunning[200];
+static int numIdle[200];
+static int numHosts = 0;
+#ifdef FRACTIME
+struct EventTime hEpoch;
+struct EventTime hEndOfTime;
+#else
+static long hEpoch;
+static long hEndOfTime;
+#endif
+
+static Tree stateTree;
+static TreeNode lastEvent = NULL;
+
+int
+findHost (machine)
+char *machine;
+{
+	int i;
+	if (strcmp (machine, "x") == 0)
+		return 199;
+	for (i = 0; i < numHosts; ++i)
+		if (strcmp (machine, hostNames[i]) == 0)
+			return i;
+	return 199;
+}
+
+static int
+compareEvents (a, b)
+Key *a, *b;
+{
+	struct Event *ax, *bx;
+	ax = (struct Event *) a->pkey;
+	bx = (struct Event *) b->pkey;
+#ifdef FRACTIME
+	if (ax->time.secs == bx->time.secs)
+		return (ax->time.millisecs - bx->time.millisecs);
+	return (ax->time.secs - bx->time.secs);
+#else
+	if (ax->time == bx->time)
+		return (ax->fractime - bx->fractime);
+	return (ax->time - bx->time);
+#endif
+}
+
+static int
+computeState (hostNumber)
+{
+	if (numRunning[hostNumber] > 0)
+		return HISTO_RUNNING;
+	else if (numIdle[hostNumber] > 0)
+		return HISTO_IDLE;
+	else
+		return HISTO_OFF;
+}
+
+static void
+saveState (time, fractime, hostNumber)
+long time, fractime;
+int hostNumber;
+{
+	struct Event *foo = (struct Event *) malloc (sizeof (struct Event));
+	Key k;
+
+#ifdef FRACTIME
+	foo->time.secs = time;
+	foo->time.millisecs = fractime;
+#else
+	foo->time = time;
+	foo->fractime = fractime;
+#endif
+	foo->hostno = hostNumber;
+	foo->state = computeState (hostNumber);
+	k.pkey = (void *) foo;
+	rb_Insert (stateTree, &k, foo, compareEvents);
+}
+
+
+static void
+ScanTraceFile (filename)
+char *filename;
+{
+    FILE *fp;
+	char buf[256];
+	char event_type[50];
+	char nodetype[50];
+	char machine[50];
+	long timestamp, serial;
+	int x;
+	int oldState;
+
+	numHosts = 0;
+
+	stateTree = rb_MakeTree ();
+    if ((fp = fopen (filename, "r")) == NULL) {
+#ifdef FRACTIME
+		hEpoch.secs = 0L;
+		hEpoch.millisecs = 0;
+		hEndOfTime = hEpoch;
+#else
+		hEpoch = 0L;
+		hEndOfTime = 0L;
+#endif
 		return;
+    }
+	while (1) {
+		if (fgets (buf, sizeof (buf), fp) == NULL) {
+			msg_Format ("%s: premature end of file\n", filename);
+			return;
+		}
+		if (*buf != '#') {
+			msg_Format ("%s: bad trace file format\n", filename);
+			return;
+		}
+		sscanf(buf + 1, "%ld %ld %*d/%*d %s %s %s %*c %*d %*d",
+			   &timestamp, &serial, nodetype, machine, event_type);
 
-
-	switch (tvp->evtype) {
-
-	case TEV_START:
-		/* printf ("tev_start\n"); */
-		hmap_ResetHosts ();
-		break;
-
-	case TEV_WAITING:
-		break;
-
-	case TEV_READY:
-		n->state = ST_READY;
-		xgr_DrawNode(graphWin, n, 1);
-		break;
-
-	case TEV_RUNNING:
-		hmap_HostEvent (machine, n->state, ST_RUNNING);
-		n->state = ST_RUNNING;
-		xgr_DrawNode(graphWin, n, 1);
-		break;
-
-	case TEV_DONE:
-		hmap_HostEvent (machine, n->state, ST_DONE);
-		n->state = ST_DONE;
-		xgr_DrawNode(graphWin, n, 1);
-		break;
-
-	case TEV_DEAD:
-		hmap_HostEvent (machine, n->state, ST_DEAD);
-		n->state = ST_DEAD;
-		xgr_DrawNode(graphWin, n, 1);
-		break;
-
-	case TEV_FINISH:
-		/* printf ("tev_finish\n"); */
-		hmap_ResetHosts ();
-		gr_ResetGraph (tracegraph, ST_DEAD);
-		break;
-
-	case TEV_MACHINE:
-		/*
-		 * The tracefile spits out a list of machines before doing
-		 * any computation.  That way we don't have to define a cost
-		 * matrix file to look at a trace, and the hosts map and
-		 * (eventually) usage graph only reflect the hosts that were
-		 * used in the computation.
-		 */
-		hmap_AddHost (machine);
-		break;
-
-	case TEV_VNODE:
-	case TEV_UNKNOWN:	/* XXX a hack that probably means virt node exp */
-		switch (n->node_type) {
-
-		case NODE_LOOP:
-			if (tvp->tf) {	/* loop happens */
-				Node pairn = n->pair;
-				Node nn;
-				Tree ctree = pairn->children;
-				TreeNode ctn;
-				Node cn;
-
-	/* make copy of loop graph */
-				nn = gr_CopySubgraph(grf, n, 1,
-					pairn->xy.x - n->xy.x,
-					-(n->xy.y - pairn->xy.y + 20)); /* XXX 20 is a hack */
-				nn = nn->pair;
-
-	/* move arcs from end of old loop to end of new */
-				while ((ctn = rb_First(ctree)) != ctree) {
-					cn = (Node)rb_Value(ctn);
-					gr_RemAnArc(grf, pairn, cn);
-					gr_BuildAnArc(grf, nn, cn);
-				}
-	/* link old loop end to new loop begin */
-				nn = nn->pair;
-				gr_BuildAnArc(grf, pairn, nn);
-	/* remove old loop virtual nodes */
-				gr_NullifyNode(grf, n);
-				gr_NullifyNode(grf, pairn);
-				XClearWindow(xDisp, graphWin);
-				trace_refresh();
-
-			} else {	/* loop doesn't happen */
-	/* short ckt loop body */
-				gr_NullifySubgraph(grf, n);
-				XClearWindow(xDisp, graphWin);
-				trace_refresh();
+		if (strcmp (event_type, "MACHINE") == 0) {
+			hostNames[numHosts++] = strsave (machine);
+		}
+		else if (strcmp (event_type, "START") == 0) {
+#ifdef FRACTIME
+			hEpoch.secs = timestamp;
+			hEpoch.millisecs = serial;
+#else
+			hEpoch = timestamp;
+#endif
+			if (monitorMode) {
+#ifdef FRACTIME
+				hEndOfTime.secs = timestamp;
+				hEndOfTime.millisecs = serial;
+#else
+				hEndOfTime = timestamp;
+#endif
 			}
-			break;
-
-		case NODE_FANOUT:
-			{
-				int cnt = tvp->hi - tvp->lo;
-				int fv;
-				Node nn;
-				Node pairn = n->pair;
-
-				for (fv = 1; fv < cnt; fv++) {
-					gr_CopySubgraph(grf, n, 0, fv * 50, 0);
-				}
-				gr_NullifyNode(grf, n);
-				gr_NullifyNode(grf, pairn);
-				XClearWindow(xDisp, graphWin);
-				trace_refresh();
+		}
+		else if (strcmp (event_type, "RUNNING") == 0) {
+			if ((x = findHost (machine)) != 199) {
+				oldState = computeState (x);
+				numRunning[x]++;
+				if (oldState != computeState (x))
+					saveState (timestamp, serial, x);
+			}
+		}
+		else if (strcmp (event_type, "DONE") == 0) {
+			if ((x = findHost (machine)) != 199){
+				oldState = computeState (x);
+				--numRunning[x];
+				++numIdle[x];
+				if (oldState != computeState (x))
+					saveState (timestamp, serial, x);
+			}
+		}
+		else if (strcmp (event_type, "DEAD") == 0) {
+			if ((x = findHost (machine)) != 199) {
+				oldState = computeState (x);
+				--numIdle[x];
+				if (oldState != computeState (x))
+					saveState (timestamp, serial, x);
+			}
+		}
+		else if (strcmp (event_type, "FINISH") == 0) {
+			if (!monitorMode) {
+#ifdef FRACTIME
+				hEndOfTime.secs = timestamp;
+				hEndOfTime.millisecs = serial;
+#else
+				hEndOfTime = timestamp;
+#endif
 			}
-			break;
-
-		default:
 			break;
 		}
-		break;
-
-	default:
-		break;
 	}
+	fclose (fp);
+	return;
 }
+
+int
+getNextEvent (nextEvent)
+struct Event *nextEvent;
+{
+	TreeNode x;
+	struct Event *ptr;
+
+	if (lastEvent == NULL) {
+		abort ();
+	}
+	x = rb_Next (lastEvent);
+	if (x == stateTree)
+		return -1;
+	ptr = (struct Event *) rb_Value (x);
+	*nextEvent = *ptr;
+	lastEvent = x;
+}
+
+void
+getState (stateArray, nhosts, timep)
+struct Event *stateArray;
+int nhosts;
+#ifdef FRACTIME
+struct EventTime *timep;
+#else
+long *timep;
 #endif
+{
+	Key k;
+	struct Event x, *p;
+	TreeNode tn;
+	int fnd = 0;
+	int hostCount = nhosts;
+	int i;
+
+#ifdef FRACTIME
+	x.time = *timep;
+	if (--x.time.millisecs < 0) {
+		x.time.secs--;
+		x.time.millisecs = 999;
+	}
+#else
+	x.time = *timep - 1;
+	x.fractime = 0x7fffffff;
+#endif
+	x.hostno = 0;
+	k.pkey = &x;
+	tn = rb_Find (stateTree, &k, compareEvents, &fnd);
+	lastEvent = rb_Prev (tn);
+
+	for (i = 0; i < nhosts; ++i) {
+		stateArray[i].state = -1;
+		stateArray[i].hostno = i;
+#ifdef FRACTIME
+		stateArray[i].time.secs = 0;
+		stateArray[i].time.millisecs = 0;
+#else
+		stateArray[i].time = 0;
+		stateArray[i].fractime = 0;
+#endif
+	}
+	do {
+		if (tn == stateTree)
+			break;
+		if ((tn = rb_Prev (tn)) == stateTree)
+			break;
+		p = (struct Event *) rb_Value (tn);
+		if (p->hostno < nhosts) {
+			if (stateArray[p->hostno].state == -1) {
+				stateArray[p->hostno].state = p->state;
+				stateArray[p->hostno].time = p->time;
+#ifndef FRACTIME
+				stateArray[p->hostno].fractime = 0;
+#endif
+				--hostCount;
+			}
+		}
+	} while (hostCount > 0);
+
+	for (i = 0; i < nhosts; ++i)
+		if (stateArray[i].state == -1)
+			stateArray[i].state = HISTO_OFF;
+}
+
+setUpHistogram ()
+{
+	ScanTraceFile (traceFileName);
+	hist_Create (usagePop, hostNames, numHosts);
+	hist_SetEpoch (&hEpoch);
+	hist_SetEndOfTime (&hEndOfTime);
+}
+
+#endif /* FAKE_DATA  */
+#endif /* USAGE_GRAPH */
 
 /*
  * Local variables:
