@@ -7,7 +7,10 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: util.c,v $
+ * Revision 1.1  1992/04/08  05:50:01  moore
+ * initial RCS version
+ *
  *
  */
 
@@ -15,6 +18,23 @@ $Log$
 #include <stdio.h>
 #include <strings.h>
 #include <ctype.h>
+/*
+ * FIX for brain-damage induced by IBM and others who insist on
+ * using #define to alias index to strchr, etc.
+ */
+#ifdef rindex
+#undef rindex
+#endif
+#ifdef index
+#undef index
+#endif
+#ifdef strchr
+#undef strchr
+#endif
+#ifdef strrchr
+#undef strrchr
+#endif
+
 #include "comn.h"
 
 /*	acav()
