@@ -8,7 +8,18 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: widmain.c,v $
+ * Revision 1.4  1992/12/09  05:06:57  moore
+ * add #undefs for rindex, index, strchr, strrchr to fix IBM brain damage.
+ * tweak some type declarations, add some casts, to make alpha happy.
+ *
+ * Revision 1.3  1992/05/19  05:39:49  moore
+ * if index and rindex are defined as macros, undef them.  See the note
+ * in the change history for tilde.c
+ *
+ * Revision 1.2  1992/04/08  06:50:50  moore
+ * incorporates changes up to 8 April 1992
+ *
  *
  */
 
@@ -16,6 +27,22 @@ $Log$
 #include <stdio.h>
 #include <strings.h>
 #include <errno.h>
+/*
+ * FIX for brain-damage induced by IBM and others who insist on
+ * using #define to alias index to strchr, etc.
+ */
+#ifdef rindex
+#undef rindex
+#endif
+#ifdef index
+#undef index
+#endif
+#ifdef strchr
+#undef strchr
+#endif
+#ifdef strrchr
+#undef strrchr
+#endif
 
 #include "xincl.h"
 #include "FileSelect.h"
@@ -30,6 +57,13 @@ $Log$
 #include "pvmglue.h"
 #include "subproc.h"
 
+#ifdef index
+#undef index
+#endif
+#ifdef rindex
+#undef rindex
+#endif
+
 #ifndef R_OK				/* get *_OK constants for access call */
 #ifdef IMA_PMAX
 #include <unistd.h>
@@ -41,7 +75,9 @@ $Log$
 #endif /*IMA_PMAX*/
 #endif /*R_OK*/
 
+#ifndef errno
 extern int errno;				/* 4.3bsd needs this */
+#endif
 
 /*******************
 *  from elsewhere  *
@@ -646,7 +682,7 @@ legend_cb(w, cli, cd)
 	if (legendVisible = !legendVisible) {
 		if (!legendPop) {
 			legendPop = XtCreatePopupShell("legend", topLevelShellWidgetClass,
-				topLevel, NULL, NULL);
+				topLevel, NULL, 0);
 
 			n = 0;
 			XtSetArg(args[n], XtNheight, (XtArgVal)(10*19 + 50)); n++;
@@ -687,6 +723,7 @@ legend_cb(w, cli, cd)
  */
 
 #if 1
+static void
 directorySink (fd, buf)
 int fd;
 char *buf;
@@ -703,6 +740,7 @@ char *buf;
 	*buf = '\0';
 }
 
+static void
 gotDirectory (pid, exitArg, ptr, howDied)
 int pid;
 int exitArg;
@@ -715,6 +753,26 @@ enum causeOfDeath howDied;
 	if (currentDirectory)
 		free (currentDirectory);
 	currentDirectory = strsave (ptr);
+#if 0
+	if (ptr != &directory[0])
+		fprintf (stderr, "gotDirectory: ptr != directory\n");
+    fprintf (stderr, "directory is %s\n", directory);
+    fprintf (stderr, "ptr is %s (%x)\n", ptr, ptr);
+#endif
+#if 1
+	setenv ("PWD", currentDirectory, 1);
+#endif
+#if 0
+	if (ptr != &directory[0])
+		fprintf (stderr, "gotDirectory: ptr != directory\n");
+    fprintf (stderr, "directory is %s\n", directory);
+    fprintf (stderr, "ptr is %s (%x)\n", ptr, ptr);
+	{
+		char **ptr;
+		for (ptr = environ; *ptr; ++ptr)
+			fprintf (stderr, "%s\n", *ptr);
+	}
+#endif
 	msg_Format ("current directory is now \"%s\"\n", currentDirectory);
 	p = rindex (ptr, '/');
 	sprintf (buf, "directory: %s", p ? p+1 : buf);
