@@ -8,7 +8,13 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: xgraph.c,v $
+ * Revision 1.2  1992/09/14  19:07:16  moore
+ * allow graph text font to be set with htool.graphFont resource.
+ *
+ * Revision 1.1  1992/04/08  05:50:56  moore
+ * initial RCS version
+ *
  *
  */
 
@@ -125,9 +131,25 @@ xgr_Init()
 {
 	XGCValues xgcvalues;
 
+	static struct foo {
+		Font graphFont;
+	} defaults ;
+
+#define offset(field) XtOffsetOf (struct foo, field)
+
+	static XtResource resourceSpecs [] = {
+		{ "graphFont", XtCFont, XtRFont, sizeof(Font),
+			  offset (graphFont), XtRString, XtDefaultFont },
+	};
+
+#undef offset
+
 	if (bp_pm)
 		return;
 
+	XtGetApplicationResources (topLevel, &defaults, resourceSpecs,
+							   XtNumber (resourceSpecs), NULL, 0);
+
 	compose_help_pm = XCreatePixmapFromBitmapData(xDisp, xRootW,
 		compose_help_bits, compose_help_width, compose_help_height, 1, 0, 1);
 
@@ -165,73 +187,95 @@ xgr_Init()
 		no_error_ic_bits, no_error_ic_width, no_error_ic_height, 1, 0, 1);
 
 	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
 	xgcvalues.background = (Pixel)mycolor[COLBGND];
 	xgcvalues.foreground = (Pixel)mycolor[COLFGND];
-	gcs[GCNORMAL] = XCreateGC(xDisp, xRootW,
-		GCFunction|GCForeground|GCBackground, &xgcvalues);
+	gcs[GCNORMAL] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
 	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
 	xgcvalues.foreground = (Pixel)mycolor[COLBGND];
 	xgcvalues.background = (Pixel)mycolor[COLFGND];
-	gcs[GCINVERSE] = XCreateGC(xDisp, xRootW,
-		GCFunction|GCForeground|GCBackground, &xgcvalues);
+	gcs[GCINVERSE] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
 	xgcvalues.function = GXxor;
+	xgcvalues.font = defaults.graphFont;
 	xgcvalues.foreground = (Pixel)mycolor[COLBGND] ^ (Pixel)mycolor[COLFGND];
 	xgcvalues.background = 0;
-	gcs[GCXOR] = XCreateGC(xDisp, xRootW, GCFunction|GCForeground|GCBackground,
-		&xgcvalues);
+	gcs[GCXOR] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
 	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
 	xgcvalues.foreground = (Pixel)mycolor[COLBGND];
 	xgcvalues.background = (Pixel)mycolor[COLBGND];
-	gcs[GCCLEAR] = XCreateGC(xDisp, xRootW,
-		GCFunction|GCForeground|GCBackground, &xgcvalues);
+	gcs[GCCLEAR] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
 /*
 	if (inColor) {
 */
-        xgcvalues.function = GXcopy;
-        xgcvalues.background = (Pixel)mycolor[COLBGND];
-        xgcvalues.foreground = (Pixel)mycolor[COLNOTREADY];
-        gcs[GCNOTREADY] = XCreateGC(xDisp, xRootW,
-            GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLNOTREADY];
+	gcs[GCNOTREADY] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLERROR];
-		gcs[GCERROR] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLERROR];
+	gcs[GCERROR] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLWARNING];
-		gcs[GCWARNING] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLWARNING];
+	gcs[GCWARNING] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLSTARTED];
-		gcs[GCREADY] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLSTARTED];
+	gcs[GCREADY] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLEXECUTING];
-		gcs[GCRUNNING] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLEXECUTING];
+	gcs[GCRUNNING] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLCOMPLETED];
-		gcs[GCDONE] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLCOMPLETED];
+	gcs[GCDONE] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
-		xgcvalues.function = GXcopy;
-		xgcvalues.background = (Pixel)mycolor[COLBGND];
-		xgcvalues.foreground = (Pixel)mycolor[COLEXIT];
-		gcs[GCDEAD] = XCreateGC(xDisp, xRootW,
-			GCFunction|GCForeground|GCBackground, &xgcvalues);
+	xgcvalues.function = GXcopy;
+	xgcvalues.font = defaults.graphFont;
+	xgcvalues.background = (Pixel)mycolor[COLBGND];
+	xgcvalues.foreground = (Pixel)mycolor[COLEXIT];
+	gcs[GCDEAD] =
+		XCreateGC(xDisp, xRootW,
+				  GCFunction|GCFont|GCForeground|GCBackground, &xgcvalues);
 
 /*
 	}
