@@ -1,8 +1,8 @@
 /*
- * $Source$
- * $Revision$
- * $Date$
- * $Author$
+ * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/array.c,v $
+ * $Revision: 1.2 $
+ * $Date: 1992/05/19 05:41:23 $
+ * $Author: moore $
  * 
  * Array.c contains routines for dealing with array parameters and array 
  * constants. Array constants are defined in hence.h. The important thing
@@ -18,7 +18,7 @@
  */
 
 #include <stdio.h>
-#if defined(IMA_SYMM) || defined(IMA_NEXT)
+#if defined(IMA_SYMM) || defined(IMA_NEXT) || defined(IMA_RT)
 void *malloc ();
 void free ();
 void *realloc ();
