@@ -1,8 +1,8 @@
 /*
- * $Source$
- * $Revision$
- * $Date$
- * $Author$
+ * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/execute.c,v $
+ * $Revision: 1.3 $
+ * $Date: 1992/12/09 05:17:28 $
+ * $Author: moore $
  */
 
 #include <stdio.h>
@@ -16,6 +16,14 @@
 
 #define BKM
 
+#ifdef IMA_ALPHA
+#define MAX_IDS_IN_TABLE    1000
+static Node unique_id_table[MAX_IDS_IN_TABLE];
+static int next_unique_id = 0;
+static int unique_id (Node n);
+static Node node_from_unique_id (int i);
+#endif
+
 static Dlist Readyq; /* This is set to be G->readyq.  It's gross, but
                         I don't feel like passing G to every subroutine */
 
@@ -169,12 +177,16 @@ Graph G;
 
   trace_node(n);
   initsend();
+#ifdef IMA_ALPHA
+  putint (unique_id (n));
+#else
   putint( (int) n );		/* XXX AArrggh! Don't DO this!
 				 * slave uses this for a unique node id,
 				 * but we might want to move these things
 				 * around in memory some day.  That and
 				 * passing pointers over the net is heinous
 				 */
+#endif
   putint(n->id);
   putint(n->inst);
   putlstring(n->sub_name);
@@ -191,7 +203,11 @@ Graph G;
   putint(n->nargs);
   for(i = 0; i < n->nargs; i++) put_exp(n->args[i]);
   put_aa(n);
+#if 0
   snd(n->sub_name, n->pvminum, HENCE_INIT);
+#else
+  send_or_gripe (n->sub_name, n->pvminum, HENCE_INIT);
+#endif
 
   /* Flag the node as running */
 
@@ -235,7 +251,11 @@ Graph G;
 
   /* Check to make sure that the node's information is consistent */
 
+#ifdef IMA_ALPHA
+  n = node_from_unique_id (getint ());
+#else
   n = (Node) getint();
+#endif
   id = getint();
   inst = getint();
   if (id != n->id || inst != n->inst) {
@@ -336,7 +356,11 @@ Graph G;
 /*     printf("Master: sending DIE to %d/%d (%s/%d)\n", */
 /*             n->id, n->inst, n->sub_name, n->pvminum); */
 /*     fflush(stdout); */
+#if 0
     snd(n->sub_name, n->pvminum, HENCE_DIE);
+#else
+    send_or_gripe (n->sub_name, n->pvminum, HENCE_DIE);
+#endif
     n->state = DEAD;
     trace_node(n);
   } else {
@@ -388,6 +412,16 @@ Node n;
   /* Otherwise, make all the copies using copy_subgraph.  Put the heads
    * and tails in the head and tail arrays */
 
+#if 1
+  /* It's not at all clear to me that the code below the #endif works 
+   * for the case of FANOUT I = N to N.  So we have this hack. -KM
+   */
+  } else if (low + 1 == high) {
+      rb_traverse (r, n->children) {
+	  inject_param ((Node) r->v.val, n->ret_param, low);
+      }
+      remove_node_pair (n);
+#endif
   } else {
     heads = talloc(Dlist, high - low);
     tails = talloc(Dlist, high - low);
@@ -887,3 +921,54 @@ Node n;
           n->sub_name, n->machine, n->pvminum);
   nice_bail(CNULL);
 }
+
+#ifdef IMA_ALPHA
+/* 20-Aug-1992 AFV: don't pass 64 bit addresses as 32 bit ints */
+
+/* XXX when a node that is in the table is freed, it should be freed */
+/* in the unique_id_table also. */
+
+static int unique_id (n)
+Node n;
+{
+    int	i;
+
+    /* search for an entry in the table already */
+
+    for (i = 0; i < next_unique_id; i++) {
+	if (n == unique_id_table[i])
+	    return (i);
+    }
+
+    /* search for an entry that is no longer in use */
+    for (i = 0; i < next_unique_id; i++) {
+	if (unique_id_table[i] == NULL)	{
+	    unique_id_table[i] = n;
+	    return (i);
+	}
+    }
+
+    if (next_unique_id >= MAX_IDS_IN_TABLE) {
+	fprintf (stderr, 
+		 "Too many calls to unique_id: boost MAX_IDS_IN_TABLE (%d)\n",
+		 MAX_IDS_IN_TABLE);
+	exit (0);
+    }
+
+    i = next_unique_id++;
+    unique_id_table[i] = n;
+    return (i);
+}
+
+static Node node_from_unique_id (u)
+int u;
+{
+    if (u < 0 || u >= next_unique_id) {
+	fprintf (stderr, 
+		 "Couldn't find unique_id in table\n");
+	return (NULL);
+    }
+    return (unique_id_table[u]);
+}
+#endif /* IMA_ALPHA */
+
