@@ -3,10 +3,10 @@
 echo list.c 1>&2
 sed 's/^-//' >'list.c' <<'End of list.c'
 -/* 
-- * $Source: /n/fs/vd/jsp/src/jgraph/RCS/list.c,v $
-- * $Revision: 5.4 $
-- * $Date: 91/06/06 16:14:15 $
-- * $Author: jsp $
+- * $Source: /a/thud/chalk/homes/moore/src/hence/master/librb/RCS/rb.sh,v $
+- * $Revision: 1.1 $
+- * $Date: 1992/04/08 05:53:02 $
+- * $Author: moore $
 - */
 -
 -#include <stdio.h>    /* Basic includes and definitions */
@@ -752,10 +752,10 @@ End of rb.c
 echo list.h 1>&2
 sed 's/^-//' >'list.h' <<'End of list.h'
 -/* 
-- * $Source: /n/fs/vd/jsp/src/jgraph/RCS/list.h,v $
-- * $Revision: 5.4 $
-- * $Date: 91/06/06 16:14:16 $
-- * $Author: jsp $
+- * $Source: /a/thud/chalk/homes/moore/src/hence/master/librb/RCS/rb.sh,v $
+- * $Revision: 1.1 $
+- * $Date: 1992/04/08 05:53:02 $
+- * $Author: moore $
 - */
 -
 -/* This is the header file for the list manipulation routines in list.c.
